
local _, YssBossLoot = ...

if not YssBossLoot then
	_GYssBossLoot = _GYssBossLoot or {}
	YssBossLoot = _GYssBossLoot
end

local YBL_WMS = {}

local L = LibStub("AceLocale-3.0"):GetLocale("YssBossLoot", true)
local BZ = LibStub("LibBabble-Zone-3.0", true):GetUnstrictLookupTable()
local BB = LibStub("LibBabble-Boss-3.0", true):GetUnstrictLookupTable()

local function initWMS()
	YBL_WMS.Conts = {}
	YBL_WMS.rlConts = {}
	YBL_WMS.lConts = {}
	for name, Lname in pairs(YssBossLoot.MapTypes) do
		YBL_WMS.Conts[#YBL_WMS.Conts+1] = Lname
		YBL_WMS.rlConts[Lname] = name
		YBL_WMS.lConts[name] = Lname
	end
	table.sort(YBL_WMS.Conts)

end

local function UpdateZones()
	YBL_WMS.Zones = {}
	YBL_WMS.rlZones = {}
	YBL_WMS.lZones = {}
	for name, tex in pairs(YssBossLoot.Maps[YBL_WMS.currCont]) do
		YBL_WMS.Zones[#YBL_WMS.Zones+1] = BZ[name] or name
		YBL_WMS.rlZones[BZ[name] or name] = name
		YBL_WMS.lZones[name] = BZ[name] or name
	end
	table.sort(YBL_WMS.Zones)
end

local function UpdateLevels()
	local levels = YssBossLoot.Levels[YBL_WMS.currCont][YBL_WMS.currZone]
	local baselevel = YssBossLoot:GetLowestLevel(YBL_WMS.currCont, YBL_WMS.currZone)
	if type(levels) == 'table' then
		YBL_WMS.leveltable = levels
	else
		YBL_WMS.leveltable = {}
		for i=1, levels do
			YBL_WMS.leveltable[i] = i
		end
	end
	YBL_WMS.levelsnames = {}
	for i=1, #YBL_WMS.leveltable do
		YBL_WMS.levelsnames[i] = YssBossLoot:GetLevelName(YBL_WMS.currCont, YBL_WMS.currZone, YBL_WMS.leveltable[i])
		if (not YBL_WMS.currLevel) and YBL_WMS.leveltable[i] == baselevel then
			YBL_WMS.currLevel = YBL_WMS.leveltable[i]
			YBL_WMS.currLevelInt = i
		end
	end
	WorldMapLevelDropDownText:SetText(YBL_WMS.levelsnames[YBL_WMS.currLevelInt])
	WorldMapLevelDropDown:Show()
	WorldMapLevelUpButton:Show()
	WorldMapLevelDownButton:Show()
end

local function WorldMap_ClearTextures_Custom()
	OutlandButton:Hide()
	AzerothButton:Hide()
	for i=1, NUM_WORLDMAP_OVERLAYS do
		_G["WorldMapOverlay"..i]:SetTexture(nil);
	end
	for i = NUM_WORLDMAP_DETAIL_TILES + 1, NUM_WORLDMAP_DETAIL_TILES + NUM_WORLDMAP_PATCH_TILES do
		_G["WorldMapFrameTexture"..i]:SetTexture(nil);
	end
end

--WorldMapFrame_Update()
local WorldMapDetailFrame = _G['WorldMapDetailFrame']
local WorldMapFrame_Update_old = WorldMapFrame_Update
function WorldMapFrame_Update(...) --detect ingame map location
	YBL_WMS.currCont, YBL_WMS.currZone, YBL_WMS.currLevel = nil, nil, nil
	YssBossLoot:ClearBosses(WorldMapDetailFrame)
	
	WorldMapFrame_Update_old(...)
	
	local tex = WorldMapDetailTile1:GetTexture()
	if not tex then return end
	local map , level = string.match(tex, "([^\\]+)(%d+)_%d+")
	if not map then
		map = string.match(tex, "([^\\]+)%d+")
		level = 0
	end
	level = tonumber(level)

	local _, instanceType = GetInstanceInfo() 
	if instanceType ~= 'none' then
		local mapTypes = {
			['party'] = 'Dungeon',
			['raid'] = 'Raid',
			['pvp'] = 'Battlegrounds',
		}
		local mapType = mapTypes[instanceType]
		if YssBossLoot.Maps['r'..mapType][map] then
			YssBossLoot:AddBosses(WorldMapDetailFrame, mapType, YssBossLoot.Maps['r'..mapType][map], level)
			YBL_WMS.currCont = mapType
			YBL_WMS.LcurrCont = YBL_WMS.lConts[YBL_WMS.currCont]
			UpdateZones()
			--WorldMapContinentDropDownText:SetText(YBL_WMS.currCont)
			YBL_WMS.currZone = YssBossLoot.Maps['r'..mapType][map]
			YBL_WMS.LcurrZone = YBL_WMS.lZones[YBL_WMS.currZone]
			--WorldMapZoneDropDownText:SetText(YBL_WMS.currZone)
			UpdateLevels()
			YBL_WMS.currLevel = GetCurrentMapDungeonLevel()
		end
	end
end

function SetWorldMap(MapType, Map, dungeonLevel)
	local mapFileName = YssBossLoot.Maps[MapType] and YssBossLoot.Maps[MapType][Map]
	YssBossLootFrame.MapType = MapType
	YssBossLootFrame.Map = Map
	YssBossLootFrame.dungeonLevel = dungeonLevel
	if mapFileName and dungeonLevel then
		local completeMapFileName
		if ( dungeonLevel > 0 ) then
			completeMapFileName = mapFileName..dungeonLevel.."_"
		else
			completeMapFileName = mapFileName
		end
		for i=1, NUM_WORLDMAP_DETAIL_TILES do
			_G["WorldMapDetailTile"..i]:SetTexture("Interface\\WorldMap\\"..mapFileName.."\\"..completeMapFileName..i)
		end
		YssBossLoot:AddBosses(WorldMapDetailFrame, MapType, Map, dungeonLevel)
	else
		for i=1, NUM_WORLDMAP_DETAIL_TILES do
			_G["WorldMapDetailTile"..i]:SetTexture(nil)
		end
		YssBossLoot:ClearBosses(WorldMapDetailFrame)
	end
	enableLootTooltip = false
	YssBossLoot.LootFrame:Hide()
	UpdateLevels()
end

local WorldMapContinentButton_OnClick_old = WorldMapContinentButton_OnClick
function WorldMapContinentButton_OnClick(self, arg1, ...)
	if YBL_WMS.rlConts[arg1] then
		SetMapZoom(-1)
		--self:WorldMapFrame_Update()
		YBL_WMS.currCont = YBL_WMS.rlConts[arg1]
		YBL_WMS.LcurrCont = arg1
		UpdateZones()
		YBL_WMS.currZone = nil
		UIDropDownMenu_SetSelectedID(WorldMapContinentDropDown, self:GetID())
		WorldMap_ClearTextures_Custom()
	else
		YBL_WMS.currCont, YBL_WMS.currZone, YBL_WMS.currLevel = nil, nil, nil
		WorldMapContinentButton_OnClick_old(self, arg1, ...)
	end
end

local WorldMapContinentDropDownText_SetText = WorldMapContinentDropDownText.SetText
function WorldMapContinentDropDownText.SetText(self, text)
	if YBL_WMS.currCont then
		text = YBL_WMS.LcurrCont
	end
	WorldMapContinentDropDownText_SetText(self, text)
end

local info = {}
local WorldMapFrame_LoadContinents_old = WorldMapFrame_LoadContinents
function WorldMapFrame_LoadContinents(...)
	WorldMapFrame_LoadContinents_old(...)
	wipe(info)

	for i = 1, #YBL_WMS.Conts do
		info.text = YBL_WMS.Conts[i]
		info.arg1 = YBL_WMS.Conts[i]
		info.func = WorldMapContinentButton_OnClick
		info.checked = YBL_WMS.currCont == YBL_WMS.rlConts[YBL_WMS.Conts[i]]
		UIDropDownMenu_AddButton(info, level)
	end
end

local WorldMapZoneDropDownText_SetText = WorldMapZoneDropDownText.SetText
function WorldMapZoneDropDownText.SetText(self, text)
	if YBL_WMS.currZone then
		text = YBL_WMS.LcurrZone
	end
	WorldMapZoneDropDownText_SetText(self, text)
end

local function InstanceClick(button, arg1)
	YBL_WMS.currZone = arg1
	YBL_WMS.LcurrZone = YBL_WMS.lZones[YBL_WMS.currZone]
	UIDropDownMenu_SetSelectedID(WorldMapZoneDropDown, button:GetID())
	YBL_WMS.currLevel = nil
	SetWorldMap(YBL_WMS.currCont, YBL_WMS.currZone, YssBossLoot:GetLowestLevel(YBL_WMS.currCont, YBL_WMS.currZone))
end

local WorldMapFrame_LoadZones_old = WorldMapFrame_LoadZones
function WorldMapFrame_LoadZones(...)
	if YBL_WMS.currCont then
		wipe(info)
		for i = 1, #YBL_WMS.Zones do
			info.text = YBL_WMS.Zones[i]
			info.arg1 = YBL_WMS.rlZones[YBL_WMS.Zones[i]]
			info.func = InstanceClick
			info.checked = YBL_WMS.rlZones[YBL_WMS.Zones[i]] == YBL_WMS.currZone
			UIDropDownMenu_AddButton(info, level)
		end
	else
		WorldMapFrame_LoadZones_old(...)
	end
end

local WorldMapLevelDropDown_Initialize_old = WorldMapLevelDropDown_Initialize
function WorldMapLevelDropDown_Initialize(...)
	if YBL_WMS.currCont and YBL_WMS.currZone then
		wipe(info)
		if not YBL_WMS.leveltable then
			UpdateLevels()
		end
		UIDropDownMenu_ClearAll(WorldMapLevelDropDown)
		for i = 1, #YBL_WMS.leveltable do
			info.text = YBL_WMS.levelsnames[i]
			info.arg1 = YBL_WMS.leveltable[i]
			info.arg2 = YBL_WMS.levelsnames[i]
			info.func = function(button, arg1, arg2)
				YBL_WMS.currLevel = arg1
				YBL_WMS.currLevelInt = button:GetID()
				SetWorldMap(YBL_WMS.currCont, YBL_WMS.currZone, YBL_WMS.currLevel)
				WorldMapLevelDropDownText:SetText(arg2)
			end
			info.checked = i == YBL_WMS.currLevelInt
			UIDropDownMenu_AddButton(info, level)
		end
	elseif YBL_WMS.currCont then
		--WorldMapLevelDropDownText:SetText(nil)
	else
		WorldMapLevelDropDown_Initialize_old(...)
	end
end

local WorldMapLevelDropDown_Update_old = WorldMapLevelDropDown_Update
function WorldMapLevelDropDown_Update()
	WorldMapLevelDropDown_Update_old()
	if YBL_WMS.currLevel then
		UIDropDownMenu_SetSelectedID(WorldMapLevelDropDown, YBL_WMS.currLevelInt)
		WorldMapLevelDropDown:Show()
		WorldMapLevelUpButton:Show()
		WorldMapLevelDownButton:Show()
	end
end

local WorldMapLevelDropDownText_SetText = WorldMapLevelDropDownText.SetText
function WorldMapLevelDropDownText.SetText(self, text)
	if YBL_WMS.currLevel then
		text = YBL_WMS.levelsnames[YBL_WMS.currLevelInt]
	end
	WorldMapLevelDropDownText_SetText(self, text)
end

local WorldMapLevelUp_OnClick_old = WorldMapLevelUpButton:GetScript("OnClick")
WorldMapLevelUpButton:SetScript("OnClick", function(...)
	if YBL_WMS.currLevel then
		if 1 < YBL_WMS.currLevelInt then
			YBL_WMS.currLevelInt = YBL_WMS.currLevelInt - 1
			YBL_WMS.currLevel = YBL_WMS.leveltable[YBL_WMS.currLevelInt]
			WorldMapLevelDropDownText:SetText(YBL_WMS.levelsnames[YBL_WMS.currLevelInt])
		end
		SetWorldMap(YBL_WMS.currCont, YBL_WMS.currZone, YBL_WMS.currLevel)
		PlaySound("UChatScrollButton")
	else
		WorldMapLevelUp_OnClick_old(...)
	end
end)

local WorldMapLevelDown_OnClick_old = WorldMapLevelDownButton:GetScript("OnClick")
WorldMapLevelDownButton:SetScript("OnClick", function(...)
	if YBL_WMS.currLevel then
		if #YBL_WMS.leveltable > YBL_WMS.currLevelInt then
			YBL_WMS.currLevelInt = YBL_WMS.currLevelInt + 1
			YBL_WMS.currLevel = YBL_WMS.leveltable[YBL_WMS.currLevelInt]
			WorldMapLevelDropDownText:SetText(YBL_WMS.levelsnames[YBL_WMS.currLevelInt])
		end
		SetWorldMap(YBL_WMS.currCont, YBL_WMS.currZone, YBL_WMS.currLevel)
		PlaySound("UChatScrollButton")
	else
		WorldMapLevelDown_OnClick_old(...)
	end
end)

initWMS()
